<?php

namespace Myriad\Authentication\Lib\Strategies;

use Myriad\Authentication\Lib\Services\IPhoneNumberConfirmer;
use Myriad\Authentication\Lib\Strategies\IVerificationStrategy;
use Myriad\Authentication\Models\UserPhone;

class PhoneNumberVerification implements IVerificationStrategy, IPhoneNumberConfirmer
{
  use CreatesVerificationCodes;
  use VerifiesUserContactMethod;

  protected $mailer;
  protected $config;

  /**
   * @param \Myriad\Authentication\Lib\Repositories\IUserRepository $userRepository
   */
  public function __construct(
    \Myriad\Authentication\Lib\Repositories\IUserRepository $userRepository,
    \Illuminate\Contracts\Mail\Mailer $mailer,
    \Illuminate\Contracts\Config\Repository $config
  ) {
    $this->userRepository = $userRepository;
    $this->mailer = $mailer;
    $this->config = $config;

    // set the properties necessary for VerifiesUserContactMethod
    $this->model = UserPhone::class;
    $this->property = "phone";
    $this->contactCollection = "phoneNumbers";
    $this->contactAlreadyExistsError = "Phone number belongs to a different user!";
  }

  public function sendConfirmationTextMessage($phoneNumber, $user, array $attributes) {
    //TODO: for now this is just sending an email to testdevice@myriadmobile.com...not actually
    //      doing its job yet. i think all current constructor dependencies can be removed them as well
    $attributes = [
      'type' => array_key_exists('type', $attributes) ? $attributes['type'] : 'personal',
      'primary' => array_key_exists('primary', $attributes) ? $attributes['primary'] : false,
      'user_id' => is_null($user) ? null : $user->getAuthIdentifier(),
    ];

    $code = $this->createVerificationCode( $phoneNumber, PhoneNumberVerification::class, $attributes);

    $emailTemplate = $this->config->get('authentication.email.templates.email_address_verification');

    // send an email asking user to verify their code
    $this->mailer->send(
      $emailTemplate['template'],
      [
        'code' => $code->code,
        'app_name' => $this->config->get('authentication.email.app_name')
      ],
      function($message) use ($emailTemplate) {
        $message->to('testdevice@myriadmobile.com')->subject($emailTemplate['subject']);
      }
    );
  }
}
